package net.wanji.web.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpUtil;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import net.wanji.common.gts.GeometryUtil;
import net.wanji.common.gts.GeomsConvertUtil;
import net.wanji.common.gts.service.GtsService;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.databus.dao.entity.RidInfoEntity;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.dao.mapper.CrossDirDataRealtimeMapper;
import net.wanji.databus.dao.mapper.RidInfoMapper;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.databus.po.TBaseCrossInfo;
import net.wanji.web.bo.IsValidPointBO;
import net.wanji.web.bo.SpecialServiceRouteBO;
import net.wanji.web.common.enums.CrossDirEnum;
import net.wanji.web.common.enums.CrossPhasePlanTurnTypeEnum;
import net.wanji.web.common.enums.SpecialServiceCrossTurnEnum;
import net.wanji.web.common.exception.CrossRelationException;
import net.wanji.web.mapper.*;
import net.wanji.web.po.CrossDirTurnPO;
import net.wanji.web.po.RidInfoPO;
import net.wanji.web.po.SpecialServiceCrossPO;
import net.wanji.web.po.SpecialServicePO;
import net.wanji.web.service.SpecialServiceService;
import net.wanji.web.vo.RouteElementVO;
import net.wanji.web.vo.specialService.*;
import org.jetbrains.annotations.NotNull;
import org.locationtech.jts.geom.Geometry;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * @author Kent HAN
 * @date 2022/11/8 11:07
 */
@Service
public class SpecialServiceServiceImpl implements SpecialServiceService {
    @Autowired
    SpecialServiceMapper specialServiceMapper;
    @Autowired
    SpecialServiceCrossMapper specialServiceCrossMapper;
    @Autowired
    CustomRidInfoMapper customRidInfoMapper;
    @Autowired
    BaseCrossInfoMapper tBaseBaseCrossInfoMapper;
    @Autowired
    CrossDirTurnMapper crossDirTurnMapper;
    @Autowired
    CrossPhasePlanMapper crossPhasePlanMapper;
    @Autowired
    private BaseCrossInfoMapper baseCrossInfoMapper;
    @Autowired
    private RidInfoMapper ridInfoMapper;

    private Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd HH:mm:ss").create();
    @Autowired
    private GtsService gtsService;
    @Autowired
    private CrossDirDataRealtimeMapper crossDirDataRealtimeMapper;

    @Override
    public void addSpecialService(AddSpecialServiceInVO addSpecialServiceInVO) {
        SpecialServicePO specialServicePO = new SpecialServicePO();
        String name = addSpecialServiceInVO.getName();
        specialServicePO.setName(name);
        specialServiceMapper.insertOne(specialServicePO);
    }

    @Override
    public List<ListSpecialServicesOutVO> listSpecialServices(String adCode) {
        List<SpecialServicePO> specialServicePOList = specialServiceMapper.selectByAdCode(adCode);
        List<ListSpecialServicesOutVO> listSpecialServicesOutVOList = new ArrayList<>();
        for (SpecialServicePO specialServicePO : specialServicePOList) {
            ListSpecialServicesOutVO listSpecialServicesOutVO = new ListSpecialServicesOutVO();
            BeanUtils.copyProperties(specialServicePO, listSpecialServicesOutVO);
            // 构造wkt列表
            List<String> wkts = new ArrayList<>();
            Integer specialServiceId = specialServicePO.getId();
            List<SpecialServiceCrossPO> specialServiceCrossPOList =
                    specialServiceCrossMapper.selectBySpecialServiceId(specialServiceId);
            int size = specialServiceCrossPOList.size();
            for (int i = 0; i < size - 1; i++) {
                SpecialServiceCrossPO currentCross = specialServiceCrossPOList.get(i);
                SpecialServiceCrossPO nextCross = specialServiceCrossPOList.get(i + 1);
                RidInfoPO ridInfoPO = customRidInfoMapper.selectOne(currentCross.getCrossId(), nextCross.getCrossId());
                String wkt = ridInfoPO.getWkt();
                wkts.add(wkt);
            }
            listSpecialServicesOutVO.setWkts(wkts);
            listSpecialServicesOutVOList.add(listSpecialServicesOutVO);
        }
        return listSpecialServicesOutVOList;
    }

    @Override
    public void updateSpecialService(UpdateSpecialServiceInVO updateSpecialServiceInVO) {
        specialServiceMapper.updateSpecialService(updateSpecialServiceInVO);
    }

    @Override
    @Transactional
    public void deleteSpecialService(Integer id) {
        specialServiceMapper.deleteSpecialService(id);
        specialServiceCrossMapper.deleteCrossBySpecialServiceId(id);
    }

    @Override
    @Transactional
    public void addCross(AddCrossInVO addCrossInVO) throws RuntimeException{
        Integer specialServiceId = addCrossInVO.getSpecialServiceId();
        String endCrossid = addCrossInVO.getCrossId();
        SpecialServiceCrossPO specialServiceCrossPO = new SpecialServiceCrossPO();
        BeanUtils.copyProperties(addCrossInVO, specialServiceCrossPO);
        // 构造顺序号
        Integer maxSort = specialServiceCrossMapper.getMaxSort(specialServiceId);
        if (maxSort == null) {
            maxSort = 1;
        } else {
            maxSort++;
        }
        if (maxSort > 1) {
            // 插入路段长度，插入转向
            // 查询上一个路口ID
            Integer preSort = maxSort - 1;
            String startCrossid = specialServiceCrossMapper.selectCrossIdBySort(specialServiceId, preSort);
            // 查询路段
            RidInfoPO ridInfoPO = customRidInfoMapper.selectOne(startCrossid, endCrossid);
            // 路段关系不正确
            if (ridInfoPO == null) {
                throw new CrossRelationException("路段关系不正确");
            }
            // 插入路段长度
            Double roadLength = ridInfoPO.getLength();
            Double oldLength = specialServiceMapper.selectRoadLength(specialServiceId);
            oldLength += roadLength;
            specialServiceMapper.updateLength(specialServiceId, oldLength);
            // 插入上一个路口转向
            String startCrossOutDirId = ridInfoPO.getStartCrossdirid();
            Integer startCrossInDir = specialServiceCrossMapper.selectStartCrossInDir(specialServiceId, startCrossid);
            String startCrossInDirId = startCrossid + "_" + startCrossInDir + "_2_1";
            CrossDirTurnPO crossDirTurnPO = crossDirTurnMapper.selectOneByInAndOutIds(startCrossInDirId, startCrossOutDirId);
            String turntype = crossDirTurnPO.getTurntype();
            specialServiceCrossMapper.updateTurn(specialServiceId, startCrossid, turntype);
            // 更新当前路口进口
            String endDirCode = ridInfoPO.getEndDir();
            Integer endDirInt = Integer.parseInt(endDirCode);
            specialServiceCrossPO.setDir(endDirInt);
        }
        specialServiceCrossPO.setSort(maxSort);
        specialServiceCrossMapper.insertOne(specialServiceCrossPO);
    }

    @Override
    public List<CrossListOutVO> crossList(Integer specialServiceId) {
        List<SpecialServiceCrossPO> specialServiceCrossPOList =
                specialServiceCrossMapper.selectBySpecialServiceId(specialServiceId);
        List<CrossListOutVO> crossListOutVOList = new ArrayList<>();
        for (int i = 0; i < specialServiceCrossPOList.size(); i++) {
            SpecialServiceCrossPO specialServiceCrossPO = specialServiceCrossPOList.get(i);
            CrossListOutVO crossListOutVO = new CrossListOutVO();
            String crossId = specialServiceCrossPO.getCrossId();
            crossListOutVO.setCrossId(crossId);
            Integer sort = specialServiceCrossPO.getSort();
            crossListOutVO.setSort(sort);
            crossListOutVO.setStatus(specialServiceCrossPO.getStatus());
            TBaseCrossInfo tBaseCrossInfo = tBaseBaseCrossInfoMapper.selectByPrimaryKey(specialServiceCrossPO.getCrossId());
            // 经纬度
            String location = tBaseCrossInfo.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            crossListOutVO.setLongitude(lonLat[0]);
            crossListOutVO.setLatitude(lonLat[1]);
            // 路口名称
            crossListOutVO.setName(tBaseCrossInfo.getName());
            // 获取控制方向
            Integer dirCode = specialServiceCrossPO.getDir();
            String dir = CrossDirEnum.getMsgByCode(dirCode);
            String turnCode = specialServiceCrossPO.getTurn();
            if (turnCode != null) {
                String turn = SpecialServiceCrossTurnEnum.getMsgByCode(turnCode);
                String dirAndTurn = dir + turn;
                crossListOutVO.setDirAndTurn(dirAndTurn);
            }
            // 获取wkt
            if (i < specialServiceCrossPOList.size() - 1) {
                SpecialServiceCrossPO nextPO = specialServiceCrossPOList.get(i + 1);
                String startCrossid = specialServiceCrossPO.getCrossId();
                String endCrossid = nextPO.getCrossId();
                String wkt = customRidInfoMapper.selectWkt(startCrossid, endCrossid);
                crossListOutVO.setWkt(wkt);
            }
            crossListOutVOList.add(crossListOutVO);
        }

        return crossListOutVOList;
    }

    @Override
    @Transactional
    public void updateCross(UpdateCrossInVO updateCrossInVO) {
        Integer specialServiceId = updateCrossInVO.getSpecialServiceId();
        String crossId = updateCrossInVO.getCrossId();
        String dirInAndOut = updateCrossInVO.getDirInAndOut();
        // 获取方向（东西南北...）
        int length = dirInAndOut.length();
        String dirStr = dirInAndOut.substring(0, length - 2);
        Integer dir = CrossDirEnum.getCodeByMsg(dirStr);
        specialServiceCrossMapper.updatDir(specialServiceId, crossId, dir);
        // 更新当前路口转向
        Integer sort = updateCrossInVO.getSort();
        Integer nextSort = sort + 1;
        String endCrossid = specialServiceCrossMapper.selectCrossIdBySort(specialServiceId, nextSort);
        String startCrossOutDirId = customRidInfoMapper.selectStartCrossOutDir(crossId, endCrossid);
        String startCrossInDirId = crossId + "_" + dir + "_2_1";
        CrossDirTurnPO crossDirTurnPO = crossDirTurnMapper.selectOneByInAndOutIds(startCrossInDirId, startCrossOutDirId);
        String turntype = crossDirTurnPO.getTurntype();
        specialServiceCrossMapper.updateTurn(specialServiceId, crossId, turntype);
    }

    @Override
    @Transactional
    public void deleteCross(DeleteCrossInVO deleteCrossInVO) {
        String crossId = deleteCrossInVO.getCrossId();
        Integer specialServiceId = deleteCrossInVO.getSpecialServiceId();
        Integer sort = deleteCrossInVO.getSort();
        if (sort > 1) {
            Integer preSort = sort - 1;
            String preCrossid = specialServiceCrossMapper.selectCrossIdBySort(specialServiceId, preSort);
            // 减少路线长度
            Double roadLength = customRidInfoMapper.selectRoadLength(preCrossid, crossId);
            Double oldLength = specialServiceMapper.selectRoadLength(specialServiceId);
            oldLength -= roadLength;
            specialServiceMapper.updateLength(specialServiceId, oldLength);
            // 删除上一个路口转向
            specialServiceCrossMapper.updateTurn(specialServiceId, preCrossid, null);
        }
        specialServiceCrossMapper.deleteCross(specialServiceId, crossId);
    }

    @Override
    public void enableSpecialService(EnableDisableSpecialServiceInVO enableDisableSpecialServiceInVO) {
        Integer specialServiceId = enableDisableSpecialServiceInVO.getSpecialServiceId();
        specialServiceMapper.updateStatusEnable(specialServiceId);
        List<SpecialServiceCrossPO> specialServiceCrossPOList =
                specialServiceCrossMapper.selectBySpecialServiceId(specialServiceId);
        for (SpecialServiceCrossPO specialServiceCrossPO : specialServiceCrossPOList) {
            // 构造请求体
            EnableDisableSpecialServiceOutVO enableDisableSpecialServiceOutVO = new EnableDisableSpecialServiceOutVO();

            PhaseLockVO phaseLockVO = new PhaseLockVO();
            Integer dirInt = specialServiceCrossPO.getDir();
            String dirStr = dirInt.toString();
            String turnStr = specialServiceCrossPO.getTurn();
            String turnCode = CrossPhasePlanTurnTypeEnum.getCodeByMsg(turnStr);
            String phaseId = crossPhasePlanMapper.selectPhaseId(dirStr, turnCode);
            phaseLockVO.setPhaseid(phaseId);
            List<PhaseLockVO> phaseLockVOList = Arrays.asList(phaseLockVO);
            enableDisableSpecialServiceOutVO.setPhases(phaseLockVOList);
            enableDisableSpecialServiceOutVO.setCommand(1);

            String crossId = specialServiceCrossPO.getCrossId();
            // 发送请求
            String url = "http://10.102.1.182/utc/controlCommand/lockControl";
            String s = gson.toJson(enableDisableSpecialServiceOutVO);
            String response = HttpUtil.post(url, s);
            EnableDisableSpecialServiceResponseVO enableDisableSpecialServiceResponseVO =
                    gson.fromJson(response, EnableDisableSpecialServiceResponseVO.class);
            Boolean success = enableDisableSpecialServiceResponseVO.getSuccess();

            // 更新路口状态
            if (success) {
                specialServiceCrossMapper.updateStatusSuccess(specialServiceId, crossId);
            } else {
                specialServiceCrossMapper.updateStatusFail(specialServiceId, crossId);
            }
        }
    }

    @Override
    public void disableSpecialService(EnableDisableSpecialServiceInVO enableDisableSpecialServiceInVO) {
        Integer specialServiceId = enableDisableSpecialServiceInVO.getSpecialServiceId();
        specialServiceMapper.updateStatusDisable(specialServiceId);
        List<SpecialServiceCrossPO> specialServiceCrossPOList =
                specialServiceCrossMapper.selectBySpecialServiceId(specialServiceId);
        for (SpecialServiceCrossPO specialServiceCrossPO : specialServiceCrossPOList) {
            // 构造请求体
            EnableDisableSpecialServiceOutVO enableDisableSpecialServiceOutVO = new EnableDisableSpecialServiceOutVO();

            PhaseLockVO phaseLockVO = new PhaseLockVO();
            Integer dirInt = specialServiceCrossPO.getDir();
            String dirStr = dirInt.toString();
            String turnStr = specialServiceCrossPO.getTurn();
            String turnCode = CrossPhasePlanTurnTypeEnum.getCodeByMsg(turnStr);
            String phaseId = crossPhasePlanMapper.selectPhaseId(dirStr, turnCode);
            phaseLockVO.setPhaseid(phaseId);
            List<PhaseLockVO> phaseLockVOList = Arrays.asList(phaseLockVO);
            enableDisableSpecialServiceOutVO.setPhases(phaseLockVOList);
            enableDisableSpecialServiceOutVO.setCommand(0);

            String crossId = specialServiceCrossPO.getCrossId();

            // 发送请求
            String url = "http://10.102.1.182/utc/controlCommand/lockControl";
            String s = gson.toJson(enableDisableSpecialServiceOutVO);
            String response = HttpUtil.post(url, s);
            EnableDisableSpecialServiceResponseVO enableDisableSpecialServiceResponseVO =
                    gson.fromJson(response, EnableDisableSpecialServiceResponseVO.class);
            Boolean success = enableDisableSpecialServiceResponseVO.getSuccess();

            // 更新路口状态
            if (success) {
                specialServiceCrossMapper.updateStatusSuccess(specialServiceId, crossId);
            } else {
                specialServiceCrossMapper.updateStatusFail(specialServiceId, crossId);
            }
        }
    }

    @Override
    public Set<String> crossInList(String crossId) {
        List<String> crossInIdList = crossDirTurnMapper.crossInList(crossId);
        Set<String> res = new HashSet<>();
        for (String crossInId : crossInIdList) {
            String crossIn = getCrossIn(crossInId);
            res.add(crossIn);
        }
        return res;
    }

    private String getCrossIn(String crossInId) {
        // 从 13FNK0C6790_1_2_1 中获取 “北进口”
        int length = crossInId.length();
        String dir = crossInId.substring(length - 5, length - 4);
        String dirStr = CrossDirEnum.getMsgByCode(Integer.parseInt(dir));
        String s = dirStr + "进口";
        return s;
    }

    @Override
    public List<List<RouteElementVO>> specialServiceRoute(SpecialServiceRouteBO specialServiceRouteBO) {
        List<RidInfoEntity> ridInfoEntities = ridInfoMapper.selectAll();
        // 获取起点经纬度与rid连线相交的路段
        String startLonLat = specialServiceRouteBO.getStartLonLat();
        Geometry startGeometry = GeometryUtil.str2Geometry(startLonLat);
        Set<String> startRidPoints = new HashSet<>();
        for (RidInfoEntity ridInfoEntity : ridInfoEntities) {
            String ridLine = ridInfoEntity.getWkt();
            Geometry ridLineGeometry = GeometryUtil.str2Geometry(ridLine);
            Geometry ridLineBuf = GeomsConvertUtil.getGeometryBuf(ridLineGeometry, "50.0");
            boolean isIntersect = ridLineBuf.intersects(startGeometry);
            if (isIntersect) {
                String startCrossId = ridInfoEntity.getStartCrossId();
                String endCrossId = ridInfoEntity.getEndCrossId();
                startRidPoints.add(startCrossId);
                startRidPoints.add(endCrossId);
            }
        }
        // 获取起点路口
        double startMinDistance = Double.MAX_VALUE;
        BaseCrossInfoPO startCross = null;
        for (String ridPoint : startRidPoints) {
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(ridPoint);
            String location = baseCrossInfoPO.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            String ridPointLonLat = lonLat[0] + "," + lonLat[1];
            double distance = gtsService.distance(startLonLat, ridPointLonLat);
            startMinDistance = Math.min(startMinDistance, distance);
            if (distance == startMinDistance) {
                startCross = baseCrossInfoPO;
            }
        }

        // 获取终点经纬度与rid连线相交的路段
        String endLonLat = specialServiceRouteBO.getEndLonLat();
        Geometry endGeometry = GeometryUtil.str2Geometry(endLonLat);
        Set<String> endRidPoints = new HashSet<>();
        for (RidInfoEntity ridInfoEntity : ridInfoEntities) {
            String ridLine = ridInfoEntity.getWkt();
            Geometry ridLineGeometry = GeometryUtil.str2Geometry(ridLine);
            Geometry ridLineBuf = GeomsConvertUtil.getGeometryBuf(ridLineGeometry, "50.0");
            boolean isIntersect = ridLineBuf.intersects(endGeometry);
            if (isIntersect) {
                String startCrossId = ridInfoEntity.getStartCrossId();
                String endCrossId = ridInfoEntity.getEndCrossId();
                endRidPoints.add(startCrossId);
                endRidPoints.add(endCrossId);
            }
        }
        // 获取终点路口
        double endMinDistance = Double.MAX_VALUE;
        BaseCrossInfoPO endCross = null;
        for (String ridPoint : endRidPoints) {
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(ridPoint);
            String location = baseCrossInfoPO.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            String ridPointLonLat = lonLat[0] + "," + lonLat[1];
            double distance = gtsService.distance(startLonLat, ridPointLonLat);
            endMinDistance = Math.min(endMinDistance, distance);
            if (distance == endMinDistance) {
                endCross = baseCrossInfoPO;
            }
        }

        if (ObjectUtil.isNotEmpty(startCross) && ObjectUtil.isNotEmpty(endCross)
                && ObjectUtil.notEqual(startCross, endCross)) {
            List<List<RouteElementVO>> res = new ArrayList<>();
            // 构造第一条备选线路
            List<BaseCrossInfoPO> routeList1 = getRouteList(startCross, endCross);
            List<RouteElementVO> route1 = buildRoute(routeList1);
            // 构造第二条备选线路
            String startCrossId = startCross.getId();
            String endCrossLonLatStr = getCrossLonLatStr(endCross);
            List<RidInfoEntity> ridUpCrossList = ridInfoMapper.selectUpCross(startCrossId);
            Map<Double, BaseCrossInfoPO> distanceMap = new HashMap<>();
            List<Double> distanceList = new ArrayList<>();
            for (RidInfoEntity ridUpCross : ridUpCrossList) {
                String upCrossId = ridUpCross.getStartCrossId();
                BaseCrossInfoPO upCross = baseCrossInfoMapper.selectById(upCrossId);
                String upCrossLonLatStr = getCrossLonLatStr(upCross);
                double distance = gtsService.distance(endCrossLonLatStr, upCrossLonLatStr);
                distanceMap.put(distance, upCross);
                distanceList.add(distance);
            }
            Collections.sort(distanceList);
            BaseCrossInfoPO secondStartCross = distanceMap.get(distanceList.get(1));
            List<BaseCrossInfoPO> routeList2 = getRouteList(secondStartCross, endCross);
            routeList2.add(0, startCross);
            List<RouteElementVO> route2 = buildRoute(routeList2);

            res.add(route1);
            res.add(route2);
            return res;
        }
        return null;
    }

    @Override
    public void isValidPoint(IsValidPointBO isValidPointBO) {
        String lonLat = isValidPointBO.getLonLat();
        List<RidInfoEntity> ridInfoEntities = ridInfoMapper.selectAll();
        // 获取点位经纬度与rid连线相交的路段
        Geometry geometry = GeometryUtil.str2Geometry(lonLat);
        Set<String> startRidPoints = new HashSet<>();
        for (RidInfoEntity ridInfoEntity : ridInfoEntities) {
            String ridLine = ridInfoEntity.getWkt();
            Geometry ridLineGeometry = GeometryUtil.str2Geometry(ridLine);
            Geometry ridLineBuf = GeomsConvertUtil.getGeometryBuf(ridLineGeometry, "50.0");
            boolean isIntersect = ridLineBuf.intersects(geometry);
            if (isIntersect) {
                String startCrossId = ridInfoEntity.getStartCrossId();
                String endCrossId = ridInfoEntity.getEndCrossId();
                startRidPoints.add(startCrossId);
                startRidPoints.add(endCrossId);
            }
        }
        if (CollectionUtil.isEmpty(startRidPoints)) {
            throw new RuntimeException("不是有效点位");
        }
    }

    @NotNull
    private List<BaseCrossInfoPO> getRouteList(BaseCrossInfoPO startCross, BaseCrossInfoPO endCross) {
        String endCrossLonLatStr = getCrossLonLatStr(endCross);
        // 获取起点路口所有上游路口
        String startCrossId = startCross.getId();
        List<RidInfoEntity> ridUpCrossList = ridInfoMapper.selectUpCross(startCrossId);
        // 获取上有路口到终点距离最近的
        BaseCrossInfoPO nextStartCross = startCross;
        List<BaseCrossInfoPO> routeList = new ArrayList<>();
        routeList.add(nextStartCross);
        while (ObjectUtil.notEqual(endCross, nextStartCross)) {
            double minDistance = Double.MAX_VALUE;
            for (RidInfoEntity ridUpCross : ridUpCrossList) {
                String upCrossId = ridUpCross.getStartCrossId();
                BaseCrossInfoPO upCross = baseCrossInfoMapper.selectById(upCrossId);
                String upCrossLonLatStr = getCrossLonLatStr(upCross);
                double distance = gtsService.distance(endCrossLonLatStr, upCrossLonLatStr);
                minDistance = Math.min(minDistance, distance);
                if (minDistance == distance && !routeList.contains(nextStartCross)) { // 已选过的路口不可再选
                    nextStartCross = upCross;
                }
            }
            routeList.add(nextStartCross);
        }
        routeList.add(endCross);
        return routeList;
    }

    private List<RouteElementVO> buildRoute(List<BaseCrossInfoPO> routeList) {
        List<RouteElementVO> route = new ArrayList<>();
        // 遍历到倒数第二个路口
        for (int i = 0; i < routeList.size() - 1; i++) {
            RouteElementVO routeElementVO = new RouteElementVO();
            BaseCrossInfoPO firstCross = routeList.get(i);
            BaseCrossInfoPO secondCross = routeList.get(i + 1);
            String firstLonLatStr = getCrossLonLatStr(firstCross);
            String secondLonLatStr = getCrossLonLatStr(secondCross);

            routeElementVO.setStartLonLat(firstLonLatStr);
            routeElementVO.setStartName(firstCross.getName());
            routeElementVO.setEndLonLat(secondLonLatStr);
            routeElementVO.setEndName(secondCross.getName());

            // 获取路况，1为进口
            Integer status = getRidStatus(firstCross, secondCross, 1);
            routeElementVO.setStatus(status);

            route.add(routeElementVO);
        }
        return route;
    }

    private Integer getRidStatus(BaseCrossInfoPO firstCross, BaseCrossInfoPO secondCross, int inOutType) {
        String firstId = firstCross.getId();
        String secondId = secondCross.getId();
        RidInfoEntity ridInfoEntity = ridInfoMapper.selectByStartEnd(firstId, secondId);
        Integer inDir = ridInfoEntity.getInDir();
        Integer status = crossDirDataRealtimeMapper.selectStatus(secondId, inDir, inOutType);
        return status;
    }

    private String getCrossLonLatStr(BaseCrossInfoPO firstCross) {
        String firstLocation = firstCross.getLocation();
        double[] firstLonLat = CrossUtil.getLonLat(firstLocation);
        return firstLonLat[0] + "," + firstLonLat[1];
    }
}